<?php

namespace App\Nova\Lenses;

use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\Avatar;
use Laravel\Nova\Fields\Date;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Http\Requests\LensRequest;
use Laravel\Nova\Http\Requests\NovaRequest;
use Laravel\Nova\Lenses\Lens;
use Laravel\Nova\Nova;

class SoonExpiredUsers extends Lens
{
    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [];
    /**
     * Indicates whether the lens should automatically poll for new records.
     *
     * @var bool
     */
    public static $polling = false;

    /**
     * Indicates whether to show the polling toggle button inside Nova.
     *
     * @var bool
     */
    public static $showPollingToggle = false;
    /**
     * Get the query builder / paginator for the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\LensRequest  $request
     * @param  \Illuminate\Database\Eloquent\Builder  $query
     * @return mixed
     */
    public static function query(LensRequest $request, $query)
    {
        $currentDate = date('Y-m-d');
        
        return $request->withOrdering($request->withFilters(
            $query
                ->select([
                    "users.id",
                    "users.email",
                    "users.avatar",
                    "users.username",
                    "users.national_id",
                    "users.national_id_exp"
                    ])
                ->join('roles', 'users.id', 'roles.user_uuid')
                ->where("national_id_exp", "<", date('Y-m-d', strtotime($currentDate . ' +1 month')))
                ->where(function ($query){
                    $user = auth()->guard('web')->user();
                    $query->whereIn('roles.place_uuid', $user->places->pluck('id'))
                        ->orWhere(function ($query){
                            if(auth()->guard('web')->user()->superadmin){
                                $query->where('roles.user_uuid','!=','null');
                            }else{
                                $query->where('roles.user_uuid','=','null');
                            }
                        });
                          
              })
              
                
        ))->groupBy('users.id');
    }

    /**
     * Get the fields available to the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function fields(NovaRequest $request)
    {
        return [
            ID::make(Nova::__('ID'), 'id')->sortable()->hidden(),
            Avatar::make('avatar')->updateRules('nullable')->disk('userimages'),
            Text::make('Username')
                ->sortable()
                ->rules('required', 'max:255'),

            Text::make('Email')
                ->sortable()
                ->rules('required', 'email', 'max:254')
                ->creationRules('unique:users,email')
                ->updateRules('unique:users,email,{{resourceId}}'),
            Text::make('Places', function () {
                return $this->places->map(function ($place) {
                    return $place->name;
                })->join(', ');
            })->onlyOnIndex(),

            Text::make('Emirates ID','national_id')->placeholder('Emirates ID'),
            Date::make('Expiry date','national_id_exp'),
            
            Text::make('Days To Expire', function () {
                return sprintf('%s',  floor((strtotime($this->national_id_exp) - strtotime(date(now())))/3600/24));
            }),
        ];
    }

    /**
     * Get the cards available on the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function cards(NovaRequest $request)
    {
        return [];
    }

    /**
     * Get the filters available for the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function filters(NovaRequest $request)
    {
        return [];
    }

    /**
     * Get the actions available on the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function actions(NovaRequest $request)
    {
        return parent::actions($request);
    }

    /**
     * Get the URI key for the lens.
     *
     * @return string
     */
    public function uriKey()
    {
        return 'soon-expired-users';
    }
}
