<?php

namespace App\Nova;

use Illuminate\Http\Request;
use Illuminate\Validation\Rules;
use Laravel\Nova\Fields\Avatar;
use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\Date;
use Laravel\Nova\Fields\Password;
use Laravel\Nova\Fields\Text;
use Outl1ne\MultiselectField\Multiselect;
use Laravel\Nova\Http\Requests\NovaRequest;
use Laravel\Nova\Fields\BelongsToMany;
use App\Nova\Place;
use SimpleSquid\Nova\Fields\Enum\Enum;
use App\Enums\RoleType;
use Tapp\SelectCountryCode\SelectCountryCode;
use DigitalCreative\ColumnToggler\ColumnTogglerTrait;
use Laravel\Nova\Fields\PasswordConfirmation;
use YieldStudio\NovaPhoneField\PhoneNumber;

class User extends Resource
{
    use ColumnTogglerTrait;
    /**
     * The model the resource corresponds to.
     *
     * @var class-string<\App\Models\User>
     */
    public static $model = \App\Models\User::class;

    public static $trafficCop = false;

    /**
     * The single value that should be used to represent the resource when being displayed.
     *
     * @var string
     */
    public static $title = 'username';

    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [
        'id', 'username', 'email',
    ];

    /**
     * Returns the label for the profile.
     *
     * @return string The label for the profile.
     */
    public static function label() {
        return 'Profiles';
    }
    /**
     * Get the fields displayed by the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function fields(NovaRequest $request)
    {
        return [
            ID::make()->hidden(),
            Avatar::make('Avatar')->updateRules('nullable')->disk('userimages'),
            Text::make('Username')
                ->sortable()
                ->rules('required', 'max:255'),
                Text::make('Firstname')
                ->sortable()
                ->rules('max:255'),
                Text::make('Lastname')
                ->sortable()
                ->rules('max:255'),

            Text::make('Email')
                ->sortable()
                ->rules('required', 'email', 'max:254')
                ->creationRules('unique:users,email')
                ->updateRules('unique:users,email,{{resourceId}}'),
            Text::make('Places', function () {
                return $this->places->map(function ($place) {
                    return $place->name;
                })->join(', ');
            })->onlyOnIndex(),
            Password::make('Password')
            ->onlyOnForms()
            ->creationRules('required', Rules\Password::defaults(), 'confirmed')
            ->updateRules('nullable', Rules\Password::defaults(), 'confirmed'),
            PasswordConfirmation::make('Password Confirmation')
            ->onlyOnForms()
            ->creationRules('required', Rules\Password::defaults())
            ->updateRules('nullable', Rules\Password::defaults()),
            Text::make('# ID','national_id')->placeholder('Emirates ID')->hideFromIndex()->rules('nullable'),
            Date::make('Expiry date','national_id_exp')->hideFromIndex()->rules('nullable'),
            Text::make('Country Code', 'country_code')->hideFromIndex(),
            Text::make('Phone')->maxlength(15)->enforceMaxlength()->hideFromIndex(),
            PhoneNumber::make('Phone', 'mix_phone')->hideWhenCreating()->hideWhenUpdating()->hideFromDetail(),
            BelongsToMany::make('Places')->fields(function ($request, $relatedModel) {
                return [
                    MultiSelect::make('permissions')
                        ->options(RoleType::asArray())
                        ->saveAsJSON(),
                ];
            })->filterable(),
        ];
    }

    /**
     * Get the cards available for the request.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function cards(NovaRequest $request)
    {
        return [
            new Metrics\NewUsers,
        ];
    }

    /**
     * Get the filters available for the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function filters(NovaRequest $request)
    {
        return [];
    }

    /**
     * Get the lenses available for the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function lenses(NovaRequest $request)
    {
        return [
            (new Lenses\SoonExpiredUsers),
        ];
    }

    /**
     * Get the actions available for the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function actions(NovaRequest $request)
    {
        return [];
    }

    /**
     * Index the query based on the given NovaRequest and query.
     *
     * @param NovaRequest $request The NovaRequest object.
     * @param mixed $query The query to be indexed.
     * @return mixed The indexed query.
     */
    public static function indexQuery(NovaRequest $request, $query)
    {
        if($request->user()->superadmin){
            return $query;
        }

        $users = array();
        foreach($request->user()->places as $place){
            if($request->user()->places->find($place->id)->pivot->permissions->contains("MANAGER")){
                foreach($place->users as $user){
                    array_push($users, $user->id);
            }

            }
        }
        return $query
            ->whereIn('users.id', $users);

    }


}
